/**************************************************************************************************************************************************
 *       Copyright  2015 Texas Instruments Incorporated - http://www.ti.com/                                                                      *
 ***************************************************************************************************************************************************
 *  Redistribution and use in source and binary forms, with or without modification, are permitted provided that the following conditions are met: *
 *                                                                                                                                                 *
 *    Redistributions of source code must retain the above copyright notice, this list of conditions and the following disclaimer.                 *
 *                                                                                                                                                 *
 *    Redistributions in binary form must reproduce the above copyright notice, this list of conditions and the following disclaimer in the        *
 *    documentation and/or other materials provided with the distribution.                                                                         *
 *                                                                                                                                                 *
 *    Neither the name of Texas Instruments Incorporated nor the names of its contributors may be used to endorse or promote products derived      *
 *    from this software without specific prior written permission.                                                                                *
 *                                                                                                                                                 *
 *  THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT          *
 *  LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT     *
 *  OWNER OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT         *
 *  LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY    *
 *  THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE      *
 *  OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.                                                                           *
 **************************************************************************************************************************************************/

#include "msp430.h"
#include "stdint.h"
#include "swif.h"
#include "main.h"

/*************************************************************************************************************************************************
 *  initCompE
 **************************************************************************************************************************************************/
/*!
 * @brief Initialize the comparator_E
 *
 * This function is called once to configure the comparator_E
 *
 * @return  none
 **************************************************************************************************************************************************/
void initCompE(void)
{

    CECTL0 = CEIPEN | CEIPSEL_2;                                         // Enable V+, input channel CE2
    CECTL1 = CEPWRMD_0;                                                  // High-Speed Power mode
    CECTL2 = CEREF1_0 | CERS_1 | CERSEL | CEREF0_0;                      // VRef is applied to -terminal
    // VREF1 is Vcc*1/4
    // VREF0 is Vcc*3/4
    CECTL3 |= BIT1;                                                      // Input Buffer Disable @P1.1/CE1
    CECTL1 |= CEON;                                                      // Turn On ComparatorE
    __delay_cycles(CPU_CLK*75);                                          // delay for the reference to settle
    CECTL2 = CEREF1_10 | CERS_1 | CERSEL | CEREF0_18;                    // VRef is applied to -terminal

}


/*************************************************************************************************************************************************
 *  config_USCIA0_UART
 **************************************************************************************************************************************************/
/*!
 * @brief Initialize the eUSCIA0 module
 *
 * This function is called once to configure the eUSCIA0
 *
 * @param[in]   div         USCIA0 baudrate divider
 *
 * @return  none
 **************************************************************************************************************************************************/
inline void config_USCIA0_UART(uint16_t div)
{
	UCA0CTLW0 = UCSWRST;
#ifdef BREAK_SYNC_SEQUENCE
	UCA0CTLW0 |= (UCSSEL__SMCLK + UCMODE_3 + UCDORM); // + UCABDEN);
	UCA0ABCTL |= (UCDELIM0+UCDELIM1);
#else
	UCA0MCTLW = 0x0000;
	UCA0CTLW0 |= UCSSEL__SMCLK;
#endif
	UCA0BRW = div;
	P1SEL1 |= BIT1;
	UCA0CTLW0 &= ~UCSWRST;
}

/*************************************************************************************************************************************************
 *  sendByte
 **************************************************************************************************************************************************/
/*!
 * @brief send one byte
 *
 * This function is called to send the send break/synch field plus one byte
 *
 * @param[in]   x              byte to send
 *
 * @return  none
 **************************************************************************************************************************************************/
inline void sendByte(uint8_t x)
{
    while(!(UCA0IFG&UCTXIFG));
#ifdef BREAK_SYNC_SEQUENCE
	UCA0CTLW0 |= UCTXBRK;
	UCA0TXBUF = 0x55;
	while(!(UCA0IFG&UCTXCPTIFG));
	UCA0IFG &= ~UCTXCPTIFG;
#endif
	UCA0TXBUF = x;
	while(!(UCA0IFG&UCTXCPTIFG));
	UCA0IFG &= ~UCTXCPTIFG;
}

/*************************************************************************************************************************************************
 *  sendByte_RX
 **************************************************************************************************************************************************/
/*!
 * @brief send break/synch field plus one byte
 *
 * This function is called to send the send break/synch field plus one byte.
 * After transmission is completed the MSP430 is configured for receive again
 *
 * @param[in]   x              byte to send
 *
 * @return  none
 **************************************************************************************************************************************************/
inline void sendByte_RX(uint8_t x)
{
	while(!(UCA0IFG&UCTXIFG));
	UCA0CTLW0 |= UCSWRST;
	P1SEL1 |= BIT1;
	P1DIR  |= BIT1;
	CECTL0 = CEIPEN | CEIPSEL_2;
	P3SEL1 &= ~BIT5;
	CECTL3 &= ~BIT1;
	UCA0CTLW0 &= ~UCSWRST;
#ifdef BREAK_SYNC_SEQUENCE
	UCA0CTLW0 |= UCTXBRK;
	UCA0TXBUF = 0x55;
	while(!(UCA0IFG&UCTXCPTIFG));
#endif
	UCA0IFG &= ~UCTXCPTIFG;
	UCA0TXBUF = x;
	while(!(UCA0IFG&UCTXCPTIFG));
	UCA0IFG &= ~UCTXCPTIFG;
	UCA0CTLW0 |= UCSWRST;
	P1DIR  &= ~BIT1;
	P1SEL1 &= ~BIT1;
	__delay_cycles(CPU_CLK*5);
	CECTL0 = CEIPEN | CEIPSEL_1;
	P3SEL1 |= BIT5;
	CECTL3 |= BIT1;
	UCA0CTLW0 &= ~UCSWRST;
	UCA0IE |= UCRXIE;
}

/*************************************************************************************************************************************************
 *  prepareTX
 **************************************************************************************************************************************************/
/*!
 * @brief prepare the MSP430 for TX
 *
 * This function is called to prepare the MSP430 for TX mode
 *
 * @return  none
 **************************************************************************************************************************************************/
inline void prepareTX(void)
{
	UCA0CTLW0 |= UCSWRST;
	P1SEL1 |= BIT1;
	P1DIR  |=BIT1;
	CECTL0 = CEIPEN | CEIPSEL_2;
	P3SEL1 &= ~BIT5;
	CECTL3 &= ~BIT1;
	UCA0CTLW0 &= ~UCSWRST;
	//UCA0IE |= UCTXIE;
}

/*************************************************************************************************************************************************
 *  prepareRX
 **************************************************************************************************************************************************/
/*!
 * @brief prepare the MSP430 for RX
 *
 * This function is called to prepare the MSP430 for RX mode
 *
 * @return  none
 **************************************************************************************************************************************************/
inline void prepareRX(void)
{
	UCA0CTLW0 |= UCSWRST;
	P1DIR  &= ~BIT1;
	P1SEL1 &= ~BIT1;
	__delay_cycles(CPU_CLK*5);
	CECTL0 = CEIPEN | CEIPSEL_1;
	P3SEL1 |= BIT5;
	UCA0CTLW0 &= ~UCSWRST;
	CECTL3 |= BIT1;
	UCA0IE |= UCRXIE;
}

